@@ -4,17 +4,18 @@
 #' @description Checks for and locates influential observations (i.e., "outliers") via several distance and/or clustering methods. If several methods are selected, the returned "Outlier" vector will be a composite outlier score, made of the average of the binary (0 or 1) results of each method. It represents the probability of each observation of being classified as an outlier by at least one method. The decision rule used by default is to classify as outliers observations which composite outlier score is superior or equal to 0.5 (i.e., that were classified as outliers by at least half of the methods). See the \strong{Details} section below for a description of the methods.
 #'
 #' @param x A model or a data.frame object.
-#' @param method The outlier detection method(s). Can be "all" or some of c("cook", "pareto", "zscore", "iqr", "mahalanobis", "robust", "mcd", "ics", "optics", "iforest", "lof").
+#' @param method The outlier detection method(s). Can be "all" or some of c("cook", "pareto", "zscore", "iqr", "mahalanobis", "robust", "mcd", "ics", "optics", "lof").
 #' @param threshold A list containing the threshold values for each method (e.g. \code{list('mahalanobis' = 7, 'cook' = 1)}), above which an observation is
 #'   considered as outlier. If \code{NULL}, default values will be used (see 'Details').
 #'
 #' @param ... When \code{method = "ics"}, further arguments in \code{...} are
 #'   passed down to \code{ICSOutlier::ics.outlier()}.
 #'
-#' @return Check (message) on whether outliers were detected or not, as well as a
-#' data frame (with the original data that was used to fit the model), including
-#' information on the distance measure and whether or not an observation is considered
-#' as outlier.
+#' @return A logical vector of the detected outliers with a nice printing
+#'   method: a check (message) on whether outliers were detected or not. The
+#'   information on the distance measure and whether or not an observation is
+#'   considered as outlier can be recovered with the \code{as.data.frame}
+#'   function.
 #'
 #' @note There is also a \href{https://easystats.github.io/see/articles/performance.html}{\code{plot()}-method} implemented in the \href{https://easystats.github.io/see/}{\pkg{see}-package}.
 #'
@@ -44,7 +45,7 @@
 #' \subsection{Univariate methods}{
 #' \itemize{
 #' \item \strong{Z-scores}:
-#'  The Z-score, or standard score, is a way of describing a data point as deviance from a central value, in terms of standard deviations from the mean or, as it is here the case by default (Iglewicz, 1993), in terms of Median Absolute Deviation (MAD) from the median (which are robust measures of dispersion and centrality). The default threshold to classify outliers is 1.959, corresponding to the 2.5\% most extreme observations (assuming the data is normally distributed). Importantly, the Z-score method is univariate: it is computed column by column. If a data.frame is passed, then the maximum distance is kept for each observations. Thus, all observations that are extreme for at least one variable will be detected. However, this method is not suited for high dimensional data (with many columns), returning too liberal results (detecting many outliers).
+#'  The Z-score, or standard score, is a way of describing a data point as deviance from a central value, in terms of standard deviations from the mean or, as it is here the case by default (Iglewicz, 1993), in terms of Median Absolute Deviation (MAD) from the median (which are robust measures of dispersion and centrality). The default threshold to classify outliers is 1.959 (\code{threshold = list("zscore" = = 1.959)}), corresponding to the 2.5\% (\code{qnorm(0.975)}) most extreme observations (assuming the data is normally distributed). Importantly, the Z-score method is univariate: it is computed column by column. If a data.frame is passed, then the maximum distance is kept for each observations. Thus, all observations that are extreme for at least one variable will be detected. However, this method is not suited for high dimensional data (with many columns), returning too liberal results (detecting many outliers).
 #'
 #' \item \strong{IQR}:
 #'  Using the IQR (interquartile range) is a robust method developed by John Tukey, which often appears in box-and-whisker plots (e.g., in \code{geom_boxplot}). The interquartile range is the range between the first and the third quartiles. Tukey considered as outliers any data point that fell outside of either 1.5 times (the default threshold) the IQR below the first or above the third quartile. Similar to the Z-score method, this is a univariate method for outliers detection, returning outliers detected for at least one column, and might thus not be suited to high dimensional data.
@@ -147,10 +148,13 @@
 #' # fit model with outliers
 #' model <- lm(disp ~ mpg + hp, data = mt2)
 #'
-#' check_outliers(model)
-#' # plot(check_outliers(model))
+#' ol <- check_outliers(model)
+#' # plot(ol)
+#' insight::get_data(model)[ol, ]
 #'
-#' check_outliers(model, method = c("mahalabonis", "mcd"))
+#' if (require("MASS")) {
+#'   check_outliers(model, method = c("mahalabonis", "mcd"))
+#' }
 #' \dontrun{
 #' # This one takes some seconds to finish...
 #' check_outliers(model, method = "ics")
@@ -175,7 +179,7 @@ check_outliers.default <- function(x, method = c("cook", "pareto"), threshold =
 
   # Check args
   if (all(method == "all")) {
-    method <- c("zscore", "iqr", "cook", "pareto", "mahalanobis", "mcd", "ics", "optics", "iforest", "lof")
+    method <- c("zscore", "iqr", "cook", "pareto", "mahalanobis", "robust", "mcd", "ics", "optics", "iforest", "lof")
   }
   method <- match.arg(method, c("zscore", "iqr", "cook", "pareto", "mahalanobis", "robust", "mcd", "ics", "optics", "iforest", "lof"), several.ok = TRUE)
 
@@ -206,7 +210,7 @@ check_outliers.default <- function(x, method = c("cook", "pareto"), threshold =
   }
 
   # Cook
-  if ("cook" %in% c(method) & insight::model_info(x)$is_bayesian == FALSE) {
+  if ("cook" %in% c(method) & insight::model_info(x)$is_bayesian == FALSE & !inherits(x, "bife")) {
     df <- cbind(df, .check_outliers_cook(x, threshold = thresholds$cook)$data_cook)
   }
   # Pareto
@@ -221,7 +225,7 @@ check_outliers.default <- function(x, method = c("cook", "pareto"), threshold =
   df <- df[c(names(df)[names(df) != "Outlier"], "Outlier")]
 
   # Out
-  outlier <- df$Outlier
+  outlier <- df$Outlier > 0.5
 
   # Attributes
   class(outlier) <- c("check_outliers", "see_check_outliers", class(outlier))
@@ -245,10 +249,6 @@ check_outliers.numeric <- function(x, method = "zscore", threshold = NULL, ...)
 
 
 
-
-
-
-
 #' @rdname check_outliers
 #' @export
 check_outliers.data.frame <- function(x, method = "mahalanobis", threshold = NULL, ...) {
@@ -258,9 +258,9 @@ check_outliers.data.frame <- function(x, method = "mahalanobis", threshold = NUL
 
   # Check args
   if (all(method == "all")) {
-    method <- c("zscore", "iqr", "mahalanobis", "mcd", "ics", "optics", "iforest", "lof")
+    method <- c("zscore", "iqr", "cook", "pareto", "mahalanobis", "robust", "mcd", "ics", "optics", "iforest", "lof")
   }
-  method <- match.arg(method, c("zscore", "iqr", "cook", "pareto", "mahalanobis", "robust", "mcd", "ics", "optics", "iforest", "lof"), several.ok = TRUE)
+  method <- match.arg(method, c("zscore", "iqr", "cook", "pareto", "mahalanobis", "robust", "mcd", "ics", "optics", "lof"), several.ok = TRUE)
 
   # Thresholds
   if (is.null(threshold)) {
@@ -309,9 +309,9 @@ check_outliers.data.frame <- function(x, method = "mahalanobis", threshold = NUL
   }
 
   # Isolation Forest
-  if ("iforest" %in% c(method)) {
-    out <- c(out, .check_outliers_iforest(x, threshold = thresholds$iforest))
-  }
+  # if ("iforest" %in% c(method)) {
+  #   out <- c(out, .check_outliers_iforest(x, threshold = thresholds$iforest))
+  # }
 
   # Local Outlier Factor
   if ("lof" %in% c(method)) {
@@ -330,7 +330,7 @@ check_outliers.data.frame <- function(x, method = "mahalanobis", threshold = NUL
   df$Outlier <- rowMeans(df[grepl("Outlier_", names(df))])
 
   # Out
-  outlier <- df$Outlier
+  outlier <- df$Outlier > 0.5
 
   # Attributes
   class(outlier) <- c("check_outliers", "see_check_outliers", class(outlier))
@@ -356,20 +356,24 @@ as.numeric.check_outliers <- function(x, ...) {
 
 
 
+.check_outliers_thresholds <- function(x) {
+  suppressWarnings(.check_outliers_thresholds_nowarn(x))
+}
+
 #' @importFrom stats qf qchisq
-.check_outliers_thresholds <- function(x,
-                                       zscore = stats::qnorm(p = 1 - 0.025),
-                                       iqr = 1.5,
-                                       cook = stats::qf(0.5, ncol(x), nrow(x) - ncol(x)),
-                                       pareto = 0.7,
-                                       mahalanobis = stats::qchisq(p = 1 - 0.025, df = ncol(x)),
-                                       robust = stats::qchisq(p = 1 - 0.025, df = ncol(x)),
-                                       mcd = stats::qchisq(p = 1 - 0.025, df = ncol(x)),
-                                       ics = 0.025,
-                                       optics = 2 * ncol(x),
-                                       iforest = 0.025,
-                                       lof = 0.025,
-                                       ...) {
+.check_outliers_thresholds_nowarn <- function(x) {
+  zscore <- stats::qnorm(p = 1 - 0.025)
+  iqr <- 1.5
+  cook <- stats::qf(0.5, ncol(x), nrow(x) - ncol(x))
+  pareto <- 0.7
+  mahalanobis <- stats::qchisq(p = 1 - 0.025, df = ncol(x))
+  robust <- stats::qchisq(p = 1 - 0.025, df = ncol(x))
+  mcd <- stats::qchisq(p = 1 - 0.025, df = ncol(x))
+  ics <- 0.025
+  optics <- 2 * ncol(x)
+  iforest <- 0.025
+  lof <- 0.025
+
   list(
     "zscore" = zscore,
     "iqr" = iqr,
@@ -387,12 +391,13 @@ as.numeric.check_outliers <- function(x, ...) {
 
 
 
+#' @importFrom stats median sd qnorm mad
 .check_outliers_zscore <- function(x, threshold = stats::qnorm(p = 1 - 0.025), robust = TRUE, method = "max") {
   # Standardize
   if (robust == FALSE) {
-    d <- abs(as.data.frame(sapply(x, function(x) (x - mean(x, na.rm = TRUE)) / sd(x, na.rm = TRUE))))
+    d <- abs(as.data.frame(sapply(x, function(x) (x - mean(x, na.rm = TRUE)) / stats::sd(x, na.rm = TRUE))))
   } else {
-    d <- abs(as.data.frame(sapply(x, function(x) (x - median(x, na.rm = TRUE)) / mad(x, na.rm = TRUE))))
+    d <- abs(as.data.frame(sapply(x, function(x) (x - stats::median(x, na.rm = TRUE)) / stats::mad(x, na.rm = TRUE))))
   }
 
   out <- data.frame(Obs = 1:nrow(as.data.frame(d)))
@@ -525,7 +530,7 @@ as.numeric.check_outliers <- function(x, ...) {
 
 
   # Compute
-  out$Distance_Robust <- bigutilsr::covRob(x, estim = "pairwiseGK")$dist
+  out$Distance_Robust <- bigutilsr::dist_ogk(as.matrix(x))
 
   # Filter
   out$Outlier_Robust <- as.numeric(out$Distance_Robust > threshold)
@@ -648,40 +653,40 @@ as.numeric.check_outliers <- function(x, ...) {
 }
 
 
-#' @importFrom utils packageVersion
-#' @importFrom stats median qnorm mad sd
-.check_outliers_iforest <- function(x, threshold = 0.025) {
-  out <- data.frame(Obs = 1:nrow(x))
-
-  # Install packages
-  if (!requireNamespace("solitude", quietly = TRUE)) {
-    stop("Package `solitude` required for this function to work. Please install it by running `install.packages('solitude')`.", call. = FALSE)
-  }
-
-  # Compute
-  if (utils::packageVersion("solitude") < "0.2.0") {
-    iforest <- solitude::isolationForest(x)
-    out$Distance_iforest <- predict(iforest, x, type = "anomaly_score")
-  } else if (utils::packageVersion("solitude") == "0.2.0") {
-    stop("Must update package `solitude` (above version 0.2.0). Please run `install.packages('solitude')`.", call. = FALSE)
-  } else {
-    iforest <- solitude::isolationForest$new(sample_size = nrow(x))
-    suppressMessages(iforest$fit(x))
-    out$Distance_iforest <- iforest$scores$anomaly_score
-  }
-
-
-  # Threshold
-  cutoff <- stats::median(out$Distance_iforest) + stats::qnorm(1 - threshold) * stats::mad(out$Distance_iforest)
-  # Filter
-  out$Outlier_iforest <- as.numeric(out$Distance_iforest >= cutoff)
-
-  out$Obs <- NULL
-  list(
-    "data_iforest" = out,
-    "threshold_iforest" = threshold
-  )
-}
+# @importFrom utils packageVersion
+# @importFrom stats median qnorm mad sd predict
+# .check_outliers_iforest <- function(x, threshold = 0.025) {
+#   out <- data.frame(Obs = 1:nrow(x))
+#
+#   # Install packages
+#   if (!requireNamespace("solitude", quietly = TRUE)) {
+#     stop("Package `solitude` required for this function to work. Please install it by running `install.packages('solitude')`.", call. = FALSE)
+#   }
+#
+#   # Compute
+#   if (utils::packageVersion("solitude") < "0.2.0") {
+#     iforest <- solitude::isolationForest(x)
+#     out$Distance_iforest <- stats::predict(iforest, x, type = "anomaly_score")
+#   } else if (utils::packageVersion("solitude") == "0.2.0") {
+#     stop("Must update package `solitude` (above version 0.2.0). Please run `install.packages('solitude')`.", call. = FALSE)
+#   } else {
+#     iforest <- solitude::isolationForest$new(sample_size = nrow(x))
+#     suppressMessages(iforest$fit(x))
+#     out$Distance_iforest <- iforest$scores$anomaly_score
+#   }
+#
+#
+#   # Threshold
+#   cutoff <- stats::median(out$Distance_iforest) + stats::qnorm(1 - threshold) * stats::mad(out$Distance_iforest)
+#   # Filter
+#   out$Outlier_iforest <- as.numeric(out$Distance_iforest >= cutoff)
+#
+#   out$Obs <- NULL
+#   list(
+#     "data_iforest" = out,
+#     "threshold_iforest" = threshold
+#   )
+# }
 
 
 
@@ -713,7 +718,28 @@ as.numeric.check_outliers <- function(x, ...) {
 
 
 
+
+
+
+
+# Non-supported model classes ---------------------------------------
+
 #' @export
 check_outliers.glmmTMB <- function(x, ...) {
   NULL
 }
+
+#' @export
+check_outliers.lme <- function(x, ...) {
+  NULL
+}
+
+#' @export
+check_outliers.lmrob <- function(x, ...) {
+  NULL
+}
+
+#' @export
+check_outliers.glmrob <- function(x, ...) {
+  NULL
+}
