@@ -5,9 +5,9 @@
 #' different models at once and hence allows comparison of indices across models.
 #'
 #' @param ... Multiple model objects (also of different classes).
-#' @param metrics Can be \code{"all"}, \code{"common"} or a character vector of metrics to be computed. See related \link[=model_performance]{documentation} of object's class for details.
-#' @param rank Logical, if \code{TRUE}, models are ranked according to "best overall
-#'   model performance". See 'Details'.
+#' @param metrics Can be \code{"all"}, \code{"common"} or a character vector of metrics to be computed. See related \code{\link[=model_performance]{documentation}} of object's class for details.
+#' @param rank Logical, if \code{TRUE}, models are ranked according to 'best' overall model performance. See 'Details'.
+#' @param bayesfactor Logical, if \code{TRUE}, a Bayes factor for model comparisons is possibly returned. See 'Details'.
 #'
 #' @return A data frame (with one row per model) and one column per "index" (see \code{metrics}).
 #'
@@ -16,17 +16,29 @@
 #' @details \subsection{Bayes factor for Model Comparison}{
 #'   If all models were fit from the same data, \code{compare_performance()}
 #'   returns an additional column named \code{BF}, which shows the Bayes factor
-#'   (see \code{\link[bayestestR]{bayesfactor_models}}) for each model against
+#'   (see \code{bayestestR::bayesfactor_models()}) for each model against
 #'   the denominator model. The \emph{first} model is used as denominator model,
 #'   and its Bayes factor is set to \code{NA} to indicate the reference model.
 #'   }
+#'   \subsection{Likelihood-Ratio Test}{
+#'   If possible, \code{compare_performance()} conducts a likelihood-ratio test
+#'   (see \code{\link{performance_lrt}}) and adds a column with the p-values
+#'   from that test to the output. Thus, when a \code{"p"}-column is included,
+#'   this refers to the likelihood-ratio test.
+#'   }
 #'   \subsection{Ranking Models}{
 #'   When \code{rank = TRUE}, a new column \code{Performance_Score} is returned. This
 #'   score ranges from 0\% to 100\%, higher values indicating better model performance.
-#'   Calculation is based on normalizing all indices (i.e. rescaling them to a
+#'   Note that all score value do not necessarily sum up to 100\%. Rather,
+#'   calculation is based on normalizing all indices (i.e. rescaling them to a
 #'   range from 0 to 1), and taking the mean value of all indices for each model.
 #'   This is a rather quick heuristic, but might be helpful as exploratory index.
 #'   \cr \cr
+#'   When comparing frequentist models, both BIC and Bayes factor can be available.
+#'   In such cases, since Bayes factor and BIC hold the same information (i.e.
+#'   \code{cor(-$BIC, BF, method = "spearman")} is 1), the Bayes factor is ignored
+#'   for the performance-score during ranking.
+#'   \cr \cr
 #'   In particular when models are of different types (e.g. mixed models, classical
 #'   linear models, logistic regression, ...), not all indices will be computed
 #'   for each model. In case where an index can't be calculated for a specific
@@ -59,7 +71,7 @@
 #' @importFrom bayestestR bayesfactor_models
 #' @inheritParams model_performance.lm
 #' @export
-compare_performance <- function(..., metrics = "all", rank = FALSE, verbose = TRUE) {
+compare_performance <- function(..., metrics = "all", rank = FALSE, bayesfactor = TRUE, verbose = TRUE) {
   objects <- list(...)
   object_names <- match.call(expand.dots = FALSE)$`...`
 
@@ -73,20 +85,36 @@ compare_performance <- function(..., metrics = "all", rank = FALSE, verbose = TR
 
   m <- mapply(function(.x, .y) {
     dat <- model_performance(.x, metrics = metrics, verbose = FALSE)
-    cbind(data.frame(Model = as.character(.y), Type = class(.x)[1], stringsAsFactors = FALSE), dat)
+    model_name <- gsub("\"", "", .safe_deparse(.y), fixed = TRUE)
+    cbind(data.frame(Model = model_name, Type = class(.x)[1], stringsAsFactors = FALSE), dat)
   }, objects, object_names, SIMPLIFY = FALSE)
 
 
-  # check for identical model class, for bayesfactor
-  BFs <- tryCatch(
+  # likelihood ratio tests
+  LRTs <- tryCatch(
     {
-      bayestestR::bayesfactor_models(..., denominator = 1, verbose = FALSE)
+      performance_lrt(...)
     },
     error = function(e) {
       NULL
     }
   )
 
+
+  # check for identical model class, for bayesfactor
+  if (isTRUE(bayesfactor)) {
+    BFs <- tryCatch(
+      {
+        bayestestR::bayesfactor_models(..., denominator = 1, verbose = FALSE)
+      },
+      error = function(e) {
+        NULL
+      }
+    )
+  } else {
+    BFs <- NULL
+  }
+
   dfs <- Reduce(function(x, y) merge(x, y, all = TRUE, sort = FALSE), m)
 
   if (!is.null(BFs)) {
@@ -94,6 +122,12 @@ compare_performance <- function(..., metrics = "all", rank = FALSE, verbose = TR
     dfs$BF[dfs$Model == object_names[1]] <- NA
   }
 
+  if (!is.null(LRTs)) {
+    LRTs$Df <- NULL
+    LRTs$Model <- sapply(object_names, deparse)
+    dfs <- merge(dfs, LRTs, all = TRUE, sort = FALSE)
+  }
+
   # check if all models were fit from same data
   resps <- lapply(objects, insight::get_response)
   if (!all(sapply(resps[-1], function(x) identical(x, resps[[1]]))) && verbose) {
@@ -122,6 +156,12 @@ compare_performance <- function(..., metrics = "all", rank = FALSE, verbose = TR
   # set reference for Bayes factors to 1
   if ("BF" %in% colnames(x)) x$BF[is.na(x$BF)] <- 1
 
+  # don't include test statistic in ranking
+  x$p_CochransQ <- NULL
+  x$p_Omnibus <- NULL
+  x$p <- NULL
+  x$p_LRT <- NULL
+
   out <- x
 
   # normalize indices, for comparison
@@ -130,8 +170,16 @@ compare_performance <- function(..., metrics = "all", rank = FALSE, verbose = TR
     i
   })
 
+  # don't rank with BF when there is also BIC (same information)
+  if ("BF" %in% colnames(out) && "BIC" %in% colnames(out)) {
+    if (isTRUE(verbose)) {
+      message("Bayes factor is based on BIC approximation, thus BF and BIC hold the same information. Ignoring BF for performance-score.")
+    }
+    out$BF <- NULL
+  }
+
   # recode some indices, so higher values = better fit
-  for (i in c("AIC", "BIC", "RMSE")) {
+  for (i in c("AIC", "BIC", "RMSE", "Sigma")) {
     if (i %in% colnames(out)) {
       out[[i]] <- 1 - out[[i]]
     }
