@@ -1,54 +1,40 @@
-#' @importFrom insight format_table
+#' @importFrom insight export_table format_p
 #' @export
 print.compare_performance <- function(x, digits = 3, ...) {
-  insight::print_color("# Comparison of Model Performance Indices\n\n", "blue")
+  table_caption <- c("# Comparison of Model Performance Indices", "blue")
+  formatted_table <- format(x = x, digits = digits, format = "text", ...)
 
-  # if we have ranking, add score and remove incomplete indices in print
-  if ("Performance_Score" %in% colnames(x)) {
-    x$Performance_Score <- sprintf("%.2f%%", 100 * x$Performance_Score)
-    x <- x[!sapply(x, anyNA)]
+  if ("Performance_Score" %in% colnames(formatted_table)) {
+    footer <- c(sprintf("\nModel %s (of class %s) performed best with an overall performance score of %s.", formatted_table$Model[1], formatted_table$Type[1], formatted_table$Performance_Score[1]), "yellow")
+  } else {
+    footer <- NULL
   }
 
-  x[] <- lapply(x, function(i) {
-    if (is.numeric(i)) {
-      round(i, digits = digits)
-    } else {
-      i
-    }
-  })
-
-  ## TODO remove suppressWarnings() once insight is updated on CRAN (> 0.7.1)
-  cat(suppressWarnings(insight::format_table(x)))
-
-  if ("Performance_Score" %in% colnames(x)) {
-    insight::print_color(sprintf("\nModel %s (of class %s) performed best with an overall performance score of %s.\n", x$Model[1], x$Type[1], x$Performance_Score[1]), "yellow")
-  }
+  cat(insight::export_table(x = formatted_table, digits = digits, format = "text", caption = table_caption, footer = footer, ...))
+  invisible(x)
 }
 
 
 
+#' @importFrom insight export_table format_p
 #' @export
 print.performance_model <- function(x, digits = 3, ...) {
-  insight::print_color("# Indices of model performance\n\n", "blue")
-
-  x[] <- lapply(x, function(i) {
-    if (is.numeric(i)) i <- round(i, digits)
-    i
-  })
-
-  print.data.frame(x, row.names = FALSE, ...)
+  formatted_table <- format(x = x, digits = digits, format = "text", ...)
+  cat(insight::export_table(x = formatted_table, digits = digits, format = "text", caption = c("# Indices of model performance", "blue"), ...))
+  invisible(x)
 }
 
 
 #' @export
 print.check_outliers <- function(x, ...) {
-  outliers <- which(as.numeric(x) >= 0.5)
+  outliers <- which(x)
   if (length(outliers) >= 1) {
     o <- paste0(" (cases ", paste0(outliers, collapse = ", "), ")")
     insight::print_color(sprintf("Warning: %i outliers detected%s.\n", length(outliers), o), "red")
   } else {
     insight::print_color("OK: No outliers detected.\n", "green")
   }
+  invisible(x)
 }
 
 
@@ -83,6 +69,7 @@ print.check_distribution <- function(x, ...) {
 
   insight::print_color("\nPredicted Distribution of Response\n\n", "red")
   print.data.frame(x2, row.names = FALSE, ...)
+  invisible(x)
 }
 
 
@@ -98,6 +85,7 @@ print.check_distribution_numeric <- function(x, ...) {
   colnames(x1) <- c("Distribution", "Probability")
 
   print.data.frame(x1, row.names = FALSE, ...)
+  invisible(x)
 }
 
 
@@ -107,7 +95,7 @@ print.check_distribution_numeric <- function(x, ...) {
 #' @export
 print.performance_roc <- function(x, ...) {
   if (length(unique(x$Model)) == 1) {
-    cat(sprintf("AUC: %.2f%%\n", 100 * bayestestR::area_under_curve(x$Sensivity, x$Specifity)))
+    cat(sprintf("AUC: %.2f%%\n", 100 * bayestestR::area_under_curve(x$Specifity, x$Sensivity)))
   } else {
     insight::print_color("# Area under Curve\n\n", "blue")
 
@@ -119,10 +107,11 @@ print.performance_roc <- function(x, ...) {
         "  %*s: %.2f%%\n",
         max_space,
         names(dat)[i],
-        100 * bayestestR::area_under_curve(dat[[i]]$Sensivity, dat[[i]]$Specifity)
+        100 * bayestestR::area_under_curve(dat[[i]]$Specifity, dat[[i]]$Sensivity)
       ))
     }
   }
+  invisible(x)
 }
 
 
@@ -137,6 +126,7 @@ print.item_difficulty <- function(x, ...) {
   for (i in 1:length(x$item)) {
     cat(sprintf("  %*s      %.2f   %.2f\n", spaces, x$item[i], x$difficulty[i], x$ideal[i]))
   }
+  invisible(x)
 }
 
 
@@ -156,6 +146,8 @@ print.performance_pcp <- function(x, digits = 2, ...) {
 
   cat(sprintf("  Chi-squared: %*s\n", space, v1))
   cat(sprintf("      p-value: %*s\n\n", space, v2))
+
+  invisible(x)
 }
 
 
@@ -174,6 +166,7 @@ print.looic <- function(x, digits = 2, ...) {
 
   cat(out)
   cat("\n")
+  invisible(x)
 }
 
 
@@ -208,9 +201,35 @@ print.r2_generic <- function(x, digits = 3, ...) {
     out <- sprintf("  R2: %.*f", digits, x$R2)
   }
 
-
   cat(out)
   cat("\n")
+  invisible(x)
+}
+
+
+
+#' @importFrom insight print_color
+#' @export
+print.r2_mlm <- function(x, digits = 3, ...) {
+  model_type <- attr(x, "model_type")
+  if (!is.null(model_type)) {
+    insight::print_color(sprintf("# R2 for %s Regression\n\n", model_type), "blue")
+  } else {
+    insight::print_color("# R2\n\n", "blue")
+  }
+
+  for (i in names(x)) {
+    insight::print_color(sprintf("## %s\n", i), "cyan")
+    out <- paste0(c(
+      sprintf("        R2: %.*f", digits, x[[i]]$R2),
+      sprintf("   adj. R2: %.*f", digits, x[[i]]$R2_adjusted)
+    ),
+    collapse = "\n"
+    )
+    cat(out)
+    cat("\n\n")
+  }
+  invisible(x)
 }
 
 
@@ -234,6 +253,7 @@ print.r2_nakagawa <- function(x, digits = 3, ...) {
 
   cat(out)
   cat("\n")
+  invisible(x)
 }
 
 
@@ -243,57 +263,27 @@ print.r2_nakagawa <- function(x, digits = 3, ...) {
 print.r2_bayes <- function(x, digits = 3, ...) {
   insight::print_color("# Bayesian R2 with Standard Error\n\n", "blue")
 
-  out <- sprintf("  Conditional R2: %.*f [%.*f]", digits, x$R2_Bayes, digits, attr(x, "SE")[["R2_Bayes"]])
+  r2_ci <- insight::format_ci(
+    attributes(x)$CI$R2_Bayes$CI_low,
+    attributes(x)$CI$R2_Bayes$CI_high,
+    ci = attributes(x)$CI$R2_Bayes$CI / 100,
+    digits = digits
+  )
+  out <- sprintf("  Conditional R2: %.*f (%s)", digits, x$R2_Bayes, r2_ci)
 
   if ("R2_Bayes_marginal" %in% names(x)) {
-    out <- paste0(c(out, sprintf("     Marginal R2: %.*f [%.*f]", digits, x$R2_Bayes_marginal, digits, attr(x, "SE")[["R2_Bayes_marginal"]])), collapse = "\n")
+    r2_marginal_ci <- insight::format_ci(
+      attributes(x)$CI$R2_Bayes_marginal$CI_low,
+      attributes(x)$CI$R2_Bayes_marginal$CI_high,
+      ci = attributes(x)$CI$R2_Bayes_marginal$CI / 100,
+      digits = digits
+    )
+    out <- paste0(c(out, sprintf("     Marginal R2: %.*f (%s)", digits, x$R2_Bayes_marginal, r2_marginal_ci)), collapse = "\n")
   }
 
   cat(out)
   cat("\n")
-}
-
-
-
-#' @export
-print.perf_pca_rotate <- function(x, cutoff = 0.1, digits = 3, ...) {
-  .rotation <- attr(x, "rotation", exact = TRUE)
-
-  if (.rotation == "none") {
-    insight::print_color("# Loadings from Principal Component Analysis (no rotation)\n\n", "blue")
-  } else {
-    insight::print_color(sprintf("# Rotated loadings from Principal Component Analysis (%s-rotation)\n\n", .rotation), "blue")
-  }
-
-
-  xs <- attr(x, "variance", exact = TRUE)
-  x <- round(x, digits = digits)
-
-  x <- as.data.frame(apply(x, MARGIN = c(1, 2), function(.y) {
-    if (abs(.y) < cutoff) {
-      ""
-    } else {
-      as.character(.y)
-    }
-  }), stringsAsFactors = FALSE)
-
-  xs <- as.data.frame(t(as.data.frame(round(xs, digits = digits))))
-
-  colnames(xs) <- sprintf("PC%i", 1:ncol(xs))
-  rownames(xs) <- c("Proportion variance", "Cumulative variance", "Proportion explained", "Cumulative explained")
-
-  print(x, quote = FALSE, ...)
-  insight::print_color("\n(Explained) Variance\n", "cyan")
-  print(xs, ...)
-}
-
-
-
-#' @export
-print.perf_pca <- function(x, digits = 3, ...) {
-  x <- as.data.frame(round(x, digits = digits))
-  rownames(x) <- c("Standard deviation", "Eigenvalue", "Proportion variance", "Cumulative variance")
-  print(x, ...)
+  invisible(x)
 }
 
 
@@ -312,6 +302,28 @@ print.icc <- function(x, digits = 3, ...) {
 
   cat(out)
   cat("\n")
+  invisible(x)
+}
+
+
+
+#' @importFrom insight print_color
+#' @export
+print.icc_by_group <- function(x, digits = 3, ...) {
+  insight::print_color("# ICC by Group\n\n", "blue")
+  cat(insight::export_table(x, digits = digits))
+  invisible(x)
+}
+
+
+
+#' @importFrom insight print_color
+#' @export
+print.r2_nakagawa_by_group <- function(x, digits = 3, ...) {
+  insight::print_color("# Explained Variance by Level\n\n", "blue")
+  cat(insight::export_table(x, digits = digits))
+  cat("\n")
+  invisible(x)
 }
 
 
@@ -334,12 +346,16 @@ print.check_zi <- function(x, ...) {
   } else {
     message("Model seems ok, ratio of observed and predicted zeros is within the tolerance range.")
   }
+
+  invisible(x)
 }
 
 
 
 #' @export
 print.check_overdisp <- function(x, digits = 3, ...) {
+  orig_x <- x
+
   x$dispersion_ratio <- sprintf("%.*f", digits, x$dispersion_ratio)
   x$chisq_statistic <- sprintf("%.*f", digits, x$chisq_statistic)
 
@@ -362,6 +378,8 @@ print.check_overdisp <- function(x, digits = 3, ...) {
   } else {
     message("Overdispersion detected.")
   }
+
+  invisible(orig_x)
 }
 
 
@@ -457,6 +475,8 @@ print.icc_decomposed <- function(x, digits = 2, ...) {
     ci.res.lo,
     ci.res.hi
   ))
+
+  invisible(x)
 }
 
 
@@ -490,6 +510,8 @@ print.performance_hosmer <- function(x, ...) {
   } else {
     message("Summary: model does not fit well.")
   }
+
+  invisible(x)
 }
 
 
@@ -502,6 +524,8 @@ print.performance_accuracy <- function(x, ...) {
   cat(sprintf("Accuracy: %.2f%%\n", 100 * x$Accuracy))
   cat(sprintf("      SE: %.2f%%-points\n", 100 * x$SE))
   cat(sprintf("  Method: %s\n", x$Method))
+
+  invisible(x)
 }
 
 
@@ -522,6 +546,8 @@ print.performance_score <- function(x, ...) {
   cat(sprintf("logarithmic: %s\n", results[1]))
   cat(sprintf("  quadratic: %s\n", results[2]))
   cat(sprintf("  spherical: %s\n", results[3]))
+
+  invisible(x)
 }
 
 
@@ -538,6 +564,8 @@ print.check_collinearity <- function(x, ...) {
   } else {
     .print_collinearity(x)
   }
+
+  invisible(x)
 }
 
 
@@ -573,38 +601,47 @@ print.check_collinearity <- function(x, ...) {
 
 
 
-#' @importFrom insight format_table print_color
+#' @importFrom insight export_table print_color format_p
 #' @export
-print.performance_lrt <- function(x, digits = 3, ...) {
+print.performance_lrt <- function(x, digits = 2, ...) {
   insight::print_color("# Likelihood-Ratio-Test for Model Comparison\n\n", "blue")
-  cat(insight::format_table(x, digits = digits))
+
+  # value formatting
+  x$AIC <- round(x$AIC)
+  if ("BIC" %in% colnames(x)) x$BIC <- round(x$BIC)
+  x$p <- insight::format_p(x$p, name = NULL)
+
+  cat(insight::export_table(x, digits = digits))
 
   if (sum(x$p < .05, na.rm = TRUE) <= 1) {
     best <- which(x$p < .05)
     if (length(best) == 0) best <- 1
     insight::print_color(sprintf("\nModel '%s' seems to have the best model fit.\n", x$Model[best]), "yellow")
   }
+
+  invisible(x)
 }
 
 
 
-#' @importFrom insight print_color format_table
+#' @importFrom insight print_color export_table
 #' @export
 print.check_itemscale <- function(x, digits = 2, ...) {
   insight::print_color("# Description of (Sub-)Scales\n", "blue")
 
-  for (i in 1:length(x)) {
-    insight::print_color(sprintf("\nComponent %i\n\n", i), "red")
-    out <- x[[i]]
-    out[["Missings"]] <- sprintf("%.*f%%", digits, 100 * out[["Missings"]])
-    out[["Mean"]] <- sprintf("%.*f", digits, out[["Mean"]])
-    out[["SD"]] <- sprintf("%.*f", digits, out[["SD"]])
+  cat(insight::export_table(
+    lapply(1:length(x), function(i) {
+      out <- x[[i]]
+      attr(out, "table_caption") <- c(sprintf("Component %i", i), "red")
+      attr(out, "table_footer") <- c(sprintf("Mean inter-item-correlation = %.3f  Cronbach's alpha = %.3f",
+                                             attributes(out)$item_intercorrelation,
+                                             attributes(out)$cronbachs_alpha), "yellow")
 
-    cat(insight::format_table(out, missing = "<NA>"))
-
-    insight::print_color(sprintf(
-      "\nMean inter-item-correlation = %.3f  Cronbach's alpha = %.3f\n\n",
-      attributes(out)$item_intercorrelation, attributes(out)$cronbachs_alpha
-    ), "yellow")
-  }
+      out
+    }),
+    digits = digits,
+    format = "text",
+    missing = "<NA>",
+    zap_small = TRUE
+  ))
 }
